Package: aplot
Title: Decorate a 'ggplot' with Associated Information
Version: 0.2.9
Authors@R: c(
    person(given = "Guangchuang", family = "Yu", email = "guangchuangyu@gmail.com",   role  = c("aut", "cre"), comment = c(ORCID = "0000-0002-6485-8781")),
    person("Shuangbin", "Xu",       email = "xshuangbin@163.com",      role = "ctb", 
            comment = c(ORCID="0000-0003-3513-5362")),
    person(given = "Thomas", family = "Hackl", email = "thackl@mit.edu", role = "ctb")
    )
Description: For many times, we are not just aligning plots as what 'cowplot' and 'patchwork' did. Users would like to align associated information that requires axes to be exactly matched in subplots, e.g. hierarchical clustering with a heatmap. Inspired by the 'Method 2' in 'ggtree' (G Yu (2018) <doi:10.1093/molbev/msy194>), 'aplot' provides utilities to aligns associated subplots to a main plot at different sides (left, right, top and bottom) with axes exactly matched. 
Depends: R (>= 4.1.0)
Imports: ggfun (>= 0.1.3), ggplot2, ggplotify, patchwork, magrittr,
        methods, utils, yulab.utils (>= 0.1.9), pillar
Suggests: ggtree
URL: https://github.com/YuLab-SMU/aplot, https://yulab-smu.top/aplot/
License: Artistic-2.0
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-09-12 03:02:52 UTC; HUAWEI
Author: Guangchuang Yu [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6485-8781>),
  Shuangbin Xu [ctb] (ORCID: <https://orcid.org/0000-0003-3513-5362>),
  Thomas Hackl [ctb]
Maintainer: Guangchuang Yu <guangchuangyu@gmail.com>
Repository: CRAN
Date/Publication: 2025-09-12 05:30:27 UTC
Built: R 4.6.0; ; 2026-04-26 11:28:27 UTC; unix
