Package: aplot
Title: Decorate a 'ggplot' with Associated Information
Version: 0.3.1
Authors@R: c(
    person(given = "Guangchuang", family = "Yu", email = "guangchuangyu@gmail.com",   role  = c("aut", "cre"), comment = c(ORCID = "0000-0002-6485-8781")),
    person("Shuangbin", "Xu",       email = "xshuangbin@163.com",      role = "ctb", 
            comment = c(ORCID="0000-0003-3513-5362")),
    person(given = "Thomas", family = "Hackl", email = "thackl@mit.edu", role = "ctb")
    )
Description: Supports data-driven composition of a main plot with associated subplots that need precise axis alignment. Unlike general layout-focused tools such as 'cowplot' and 'patchwork', it enables related subplots to be integrated on the top, bottom, left, or right sides of a main plot with matched axes, so that the combined panels can be interpreted as a coherent whole. This design was inspired by the 'Method 2' described in 'ggtree' (G Yu (2018) <doi:10.1093/molbev/msy194>).
Depends: R (>= 4.1.0)
Imports: ggfun (>= 0.1.3), gtable, ggplot2, ggplotify, patchwork,
        magrittr, methods, utils, yulab.utils (>= 0.1.9), pillar
Suggests: ggtree, testthat (>= 3.0.0)
URL: https://github.com/YuLab-SMU/aplot, https://yulab-smu.top/aplot/
License: Artistic-2.0
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-07-07 08:04:18 UTC; HUAWEI
Author: Guangchuang Yu [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6485-8781>),
  Shuangbin Xu [ctb] (ORCID: <https://orcid.org/0000-0003-3513-5362>),
  Thomas Hackl [ctb]
Maintainer: Guangchuang Yu <guangchuangyu@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-07 08:30:02 UTC
Built: R 4.5.2; ; 2026-07-07 10:51:16 UTC; unix
