# Generated by roxygen2: do not edit by hand

S3method("<=",gglist)
S3method("[",aplot)
S3method("[<-",aplot)
S3method("[[",aplot)
S3method("[[<-",aplot)
S3method(ggplot_add,axisAlign)
S3method(grid.draw,aplot)
S3method(grid.draw,gglist)
S3method(grid.draw,oncoplot)
S3method(print,aplot)
S3method(print,gglist)
export("%>%")
export(as.patchwork)
export(gglist)
export(gglistGrob)
export(ggrange)
export(ggsave)
export(insert_bottom)
export(insert_left)
export(insert_right)
export(insert_top)
export(plot_list)
export(xlim2)
export(xrange)
export(ylim2)
export(yrange)
importFrom(ggfun,ggbreak2ggplot)
importFrom(ggfun,ggrange)
importFrom(ggfun,is.ggbreak)
importFrom(ggfun,is.ggtree)
importFrom(ggfun,xrange)
importFrom(ggfun,yrange)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_text)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggplotGrob)
importFrom(ggplot2,ggplot_add)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,is.ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_void)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggplotify,as.ggplot)
importFrom(grid,grid.draw)
importFrom(magrittr,"%>%")
importFrom(methods,is)
importFrom(patchwork,patchworkGrob)
importFrom(patchwork,plot_annotation)
importFrom(patchwork,plot_layout)
importFrom(patchwork,plot_spacer)
importFrom(utils,modifyList)
