% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide-area.R
\name{set_guide_area}
\alias{set_guide_area}
\title{set_guide_area}
\usage{
set_guide_area(
  x,
  position = c("top-right", "top-left", "bottom-right", "bottom-left", "right", "left",
    "bottom", "top")
)
}
\arguments{
\item{x}{an 'aplot' object}

\item{position}{placement of collected guides. Supported values are
`"top-right"`, `"top-left"`, `"bottom-right"`, `"bottom-left"`, `"right"`,
`"left"`, `"bottom"`, and `"top"`.
Corner positions use the largest continuous empty rectangular region
anchored at the requested corner. Side positions create a single guide
region on the requested side, where the guide box is placed as one object
rather than as repeated guide cells.}
}
\value{
an 'aplot' object
}
\description{
Set a guide area on an 'aplot' object
}
\details{
Designate a guide placement region in an 'aplot' layout for collected
guides. Corner positions use the continuous empty region anchored at the
requested corner. Side positions create a guide region on the requested side.
The selected guide area remains empty when guides are not collected.
}
